/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.test.util.query;

import com.liferay.portal.kernel.search.Hits;
import com.liferay.portal.kernel.search.SearchContext;
import com.liferay.portal.kernel.search.generic.StringQuery;
import com.liferay.portal.search.test.util.DocumentsAssert;
import com.liferay.portal.search.test.util.IdempotentRetryAssert;
import com.liferay.portal.search.test.util.indexing.BaseIndexingTestCase;
import com.liferay.portal.search.test.util.indexing.DocumentCreationHelpers;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.junit.Test;

/**
 * @author Tibor Lipusz
 * @author André de Oliveira
 */
public abstract class BaseStringQueryTestCase extends BaseIndexingTestCase {

	@Test
	public void testAnd() throws Exception {
		addDocument("description", "java eclipse");
		addDocument("description", "java liferay");
		addDocument("description", "java liferay eclipse");

		assertSearch(
			"java AND eclipse", "description",
			Arrays.asList("java eclipse", "java liferay eclipse"));

		assertSearch(
			"eclipse AND liferay", "description",
			Arrays.asList("java liferay eclipse"));
	}

	@Test
	public void testField() throws Exception {
		addDocument("title", "java");
		addDocument("title", "eclipse");
		addDocument("title", "liferay");

		assertSearch(
			"title:(java OR eclipse)", "title",
			Arrays.asList("java", "eclipse"));

		assertSearch(
			"description:(java OR eclipse)", "title", Collections.emptyList());
	}

	protected void addDocument(String fieldName, String... fieldValues)
		throws Exception {

		addDocument(DocumentCreationHelpers.singleText(fieldName, fieldValues));
	}

	protected void assertSearch(
			String query, String fieldName, List<String> expectedValues)
		throws Exception {

		SearchContext searchContext = createSearchContext();

		StringQuery stringQuery = _createStringQuery(query, searchContext);

		IdempotentRetryAssert.retryAssert(
			5, TimeUnit.SECONDS,
			() -> {
				Hits hits = search(searchContext, stringQuery);

				DocumentsAssert.assertValues(
					query, hits.getDocs(), fieldName, expectedValues);

				return null;
			});
	}

	private StringQuery _createStringQuery(
		String query, SearchContext searchContext) {

		StringQuery stringQuery = new StringQuery(query);

		stringQuery.setQueryConfig(searchContext.getQueryConfig());

		return stringQuery;
	}

}